package com.as.lyons.wcf.svc.rtn;

import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.util.Assert;

import com.as.lyons.wcf.svc.rtn.client.RTNServiceWCF;
import com.as.lyons.wcf.svc.rtn.client.IRTNService;

/**
 * 
 *
 */
@SpringBootApplication
public class App implements LogonProperties {
	private static final Logger log = LoggerFactory.getLogger(App.class);

	private static final Map<String, RtnServiceCall> svcMethods = new HashMap<>();

	private static String appStartCommand = "java -cp <...> <your class> ";

	public static void main(String[] args) {
		setAppStartCommand();
		BooleanResultServiceCall brsc = new BooleanResultServiceCall();		
		InstitutionListResultServiceCall ilrsc = new InstitutionListResultServiceCall();
		InstitutionResultServiceCall irsc = new InstitutionResultServiceCall();
		PrimaryInstitutionResultServiceCall pirc = new PrimaryInstitutionResultServiceCall();
		InstitutionListResultServiceCallBySwift ilrsswc = new InstitutionListResultServiceCallBySwift();
		svcMethods.put("VerifyRTN", brsc);
		svcMethods.put("VerifyWire", brsc);		
		svcMethods.put("GetInstitutionsDetailsPrimaryFirst", ilrsc);
		svcMethods.put("GetInstitutionsDetailsBySwift", ilrsswc);
		svcMethods.put("GetPrimaryInstitutionDetails", pirc);	
		svcMethods.put("GetInstitutionDetailsById", irsc);
		SpringApplication.run(App.class, args);
	}

	@Value("${company.id}")
	private int companyId;
	@Value("${company.user.name}")
	private String userName;
	@Value("${company.user.password}")
	private String password;

	@Bean
	public CommandLineRunner run(IRTNService svc) throws Exception {
		final LogonProperties logonProps = this;
		return args -> {
			log.info("Begin method execution");
			System.out.println("\n\n\n");
			String[] augArgs = new String[0];
			if (args != null && args.length > 0) {
				List<String> argLst = new ArrayList<>(args.length);
				for (String s : args) {
					if (s != null && !s.startsWith("--")) {
						argLst.add(s);
					}
				}
				augArgs = argLst.toArray(augArgs);
			}
			else {
				printHelp("Please provide a valid method!");
				return;
			}
			String methodStr = "";
			if(augArgs.length > 0) {
				methodStr = augArgs[0];
			}
			RtnServiceCall method = svcMethods.get(methodStr);
			if (method == null) {
				printHelp("The provided method is not recognized!");
				return;
			}
			try {							
				method.call(svc, logonProps, augArgs[0],  augArgs);
			} catch (StopProcesingException e) {
			}
			System.out.println("\n\n\n");
			log.info("End method execution");
		};
	}

	protected void printHelp(String error) {
		if (error != null) {
			System.out.println(error);
		}
		StringBuilder sb = new StringBuilder(500).append("Use: ").append(appStartCommand);
		System.out.println(sb.append("<method name> <RTN> <Country> <IncludeHistory>").toString());
		System.out.println("\twhere <method name> can be:");
		for (String method : svcMethods.keySet()) {
			System.out.println("\t\t".concat(method));
		}
		System.out.println("\n\n\n");
	}

	@Override
	public int getCompanyId() {
		return companyId;
	}

	@Override
	public String getUserName() {
		return userName;
	}

	@Override
	public String getPassword() {
		return password;
	}

	private static RTNServiceWCF createServiceWCF(String singleWsdlUrl) throws StopProcesingException {
		RTNServiceWCF wcf = null;
		try {
			wcf = new RTNServiceWCF(new URL(singleWsdlUrl));
		} catch (Exception e) {
			log.error("\nError creating the RTNServiceWCF: {}", e.getMessage());
			log.error("Exception:", e);
			throw new StopProcesingException();
		}
		return wcf;
	}

	@Bean
	private static IRTNService prepareService(@Value("${service.single.wsdl.url}") String singleWsdlUrl)
			throws StopProcesingException {
		RTNServiceWCF wcf = createServiceWCF(singleWsdlUrl);
		IRTNService svc = null;
		try {
			svc = wcf.getBasicHttpBindingIRTNService();
			Assert.notNull(svc, "The returned IRTNService is null");
		} catch (RuntimeException e) {
			System.out.println("\nError obtaining the basic HTTP binding to IRTNService: ".concat(e.getMessage()));
			e.printStackTrace();
			throw new StopProcesingException();
		}
		return svc;
	}

	private static void setAppStartCommand() {
		ClassLoader loader = App.class.getClassLoader();
		if (loader == null) {
			// Try the bootstrap classloader - obtained from the ultimate parent
			// of the System Class Loader.
			loader = ClassLoader.getSystemClassLoader();
			while (loader != null && loader.getParent() != null) {
				loader = loader.getParent();
			}
		}
		if (loader == null) {
			return;
		}
		String name = App.class.getCanonicalName();
		URL resource = loader.getResource(name.replace(".", "/") + ".class");
		if (resource == null) {
			return;
		}

		String resStr = resource.toString();
		String jar = null;
		if(resStr.startsWith("jar:")) {
			int jarIdx = resStr.indexOf(".jar!");
			if(jarIdx > 0) {
				resStr = resStr.substring(0, jarIdx + 4);
				jarIdx = resStr.lastIndexOf('/');
				if(jarIdx >= 0) {
					jar = resStr.substring(jarIdx + 1);
				} else {
					jar = resStr;
				}
			}
		}
		StringBuilder sb = new StringBuilder(500).append("java ");
		if(jar == null) {
			sb.append("-cp <...> ").append(name);
		} else {
			sb.append("-jar ").append(jar);
		}
		appStartCommand = sb.append(" ").toString();
	}
}
